<?php
class mysqlUtils {
	public static $statusReturn=array(false, "not intiated");
	public static $mysqlLink=false;
	public static $mysqlBase=false;
	function __construct($base="") {
		self::connection($base);
	}
	private static function connection($base) {
		if (file_exists(dirname(__FILE__).'/utils.php') && file_exists(dirname(__FILE__).'/mysql_credentials.php')) {
			include_once(dirname(__FILE__).'/utils.php');
			include_once(dirname(__FILE__).'/mysql_credentials.php');
			$mysqlCredentials=new mysqlCredentials();
			self::$mysqlBase=(!empty($base)?$base:$mysqlCredentials::$base);
			try {
				$dsn = 'mysql:dbname='.self::$mysqlBase.';host='.$mysqlCredentials::$host;
				$dbh = new PDO($dsn, $mysqlCredentials::$user, $mysqlCredentials::$pass);
				self::$mysqlLink=$dbh;
				self::$statusReturn=array(true, self::$mysqlLink);
				self::$mysqlLink->exec('set names utf8');
			} catch (PDOException $e) {
				self::$statusReturn=array(false, "failedToConnect", $e->getMessage());
			}
		} else {
			self::$statusReturn=array(false, "includeOfDependenciesImpossible");
		}
	}
	public static function query($query) {
		if (self::$statusReturn[0]) {
			$statement=self::$mysqlLink->prepare($query);
			$res=array();
			try {
				$statement->execute();
				$res=$statement->fetchColumn();
				return array("result"=>$res);
			} catch (PDOException $e) {
				return array("message"=>"ko","error"=>$e->getMessage());
			}
			$statement->closeCursor();
		} else {
			return self::$statusReturn;
		}
	}
	public static function selectCount($table, $field) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$sql="select count(".$field.") from ".$table;
				$statement=self::$mysqlLink->prepare($sql);
				$res=array();
				try {
					$statement->execute();
					$res=$statement->fetchColumn();
					if (preg_match('/[0-9]+/', $res)) {
						return array("message"=>"ok","count"=>$res);
					} else if (empty($res)) {
						return array("message"=>"ko","error"=>"tableDoesNotExist");
					} else {
						return array("message"=>"ko","error"=>"unknownError");
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
				$statement->closeCursor();
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function selectCountWhereString($table, $field, $whereString) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$sql="select count(".$field.") from ".$table." where ".$whereString;
				$statement=self::$mysqlLink->prepare($sql);
				$res=array();
				try {
					$statement->execute();
					$res=$statement->fetchColumn();
					if (preg_match('/[0-9]+/', $res)) {
						return array("message"=>"ok","count"=>$res);
					} else if (empty($res)) {
						return array("message"=>"ko","error"=>"tableDoesNotExist");
					} else {
						return array("message"=>"ko","error"=>"unknownError");
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
				$statement->closeCursor();
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function selectAll($table, $treatmentFunction, $sortParameters=array("boolean"=>false, "key"=>null, "sort_flags"=>null, "boolean_remap_numerical"=>false)) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$sql="select * from ".$table;
				$statement=self::$mysqlLink->prepare($sql);
				$res=array();
				try {
					$statement->execute();
					while ($r=$statement->fetch(PDO::FETCH_ASSOC)) {
						if (is_array($r)) {
							foreach($r as $k => $v) {
								if (!array_key_exists($k, $res)) {
									$res[$k]=array();
								}
								$res[$k][].=(($treatmentFunction!=null)?$treatmentFunction($v):$v);
							}
						}
					}
					unset($r);
					if ($sortParameters["boolean"]==false) {
						return $res;
					} else {
						if (is_array($sortParameters["key"])) {
							$sortParameters["key"]=array_reverse($sortParameters["key"]);
							foreach($sortParameters["key"] as $k=>$v) {
								$res=sortOn($res, $sortParameters["key"][$k], $sortParameters["sort_flags"], $sortParameters["boolean_remap_numerical"]);
							}
						}
						return $res;
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
				$statement->closeCursor();
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function selectOnFields($tableAndFields, $treatmentFunction, $sortParameters=array("boolean"=>false, "key"=>null, "sort_flags"=>null, "boolean_remap_numerical"=>false)) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $tableAndFields[0])) {
				$sql="select ";
				for ($i=0; $i<count($tableAndFields[1]); $i++) {
					$sql.=$tableAndFields[1][$i];
					if ($i<count($tableAndFields[1])-1) {
						$sql.=', ';
					}
				}
				$sql.=" from ".$tableAndFields[0];
				$statement=self::$mysqlLink->prepare($sql);
				$res=array();
				try {
					$statement->execute();
					while ($r=$statement->fetch(PDO::FETCH_ASSOC)) {
						if (is_array($r)) {
							foreach($r as $k => $v) {
								if (!array_key_exists($k, $res)) {
									$res[$k]=array();
								}
								$res[$k][].=(($treatmentFunction!=null)?$treatmentFunction($v):$v);
							}
						}
					}
					unset($r);
					if ($sortParameters["boolean"]==false) {
						return $res;
					} else {
						if (is_array($sortParameters["key"])) {
							$sortParameters["key"]=array_reverse($sortParameters["key"]);
							foreach($sortParameters["key"] as $k=>$v) {
								$res=sortOn($res, $sortParameters["key"][$k], $sortParameters["sort_flags"], $sortParameters["boolean_remap_numerical"]);
							}
						}
						return $res;
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
				$statement->closeCursor();
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function selectAllForId($table, $id, $treatmentFunction, $sortParameters=array("boolean"=>false, "key"=>null, "sort_flags"=>null, "boolean_remap_numerical"=>false)) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$statement=self::$mysqlLink->prepare("select * from ".$table." where id=:id");
				$statement->bindParam(':id', $id, PDO::PARAM_INT);
				$statement->execute();
				try {
					$r=$statement->fetch(PDO::FETCH_ASSOC);
					$res=array();
					foreach($r as $k => $v) {
						if (!array_key_exists($k, $res)) {
							$res[$k]=array();
						}
						$res[$k][].=(($treatmentFunction!=null)?$treatmentFunction($v):$v);
					}
					unset($r);
					$statement->closeCursor();
					if ($sortParameters["boolean"]==false) {
						return $res;
					} else {
						if (is_array($sortParameters["key"])) {
							$sortParameters["key"]=array_reverse($sortParameters["key"]);
							foreach($sortParameters["key"] as $k=>$v) {
								$res=sortOn($res, $sortParameters["key"][$k], $sortParameters["sort_flags"], $sortParameters["boolean_remap_numerical"]);
							}
						}
						return $res;
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function selectAllWhere($table, $whereArray, $treatmentFunction, $sortParameters=array("boolean"=>false, "key"=>null, "sort_flags"=>null, "boolean_remap_numerical"=>false)) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$intTypes=array("tinyint","smallint","mediumint","int","bigint");
				$sql="select * from ".$table." where ";
				$type=array();
				foreach ($whereArray as $k => $v) {
					$statement=self::$mysqlLink->prepare("select data_type from information_schema.columns where table_schema='".self::$mysqlBase."' and table_name=:table and column_name=:column");
					$statement->bindParam(':table', $table, PDO::PARAM_STR);
					$statement->bindParam(':column', $whereArray[$k]["name"], PDO::PARAM_STR);
					try {
						$statement->execute();
						$r=$statement->fetch(PDO::FETCH_ASSOC);
						$type[$k]=$r['data_type'];
						$statement->closeCursor();
					} catch (PDOException $e) {
						return array("message"=>"ko","error"=>$e->getMessage());
					}
					if (strlen($type[$k])>0 && in_array(strtolower($type[$k]),$intTypes)) {
						$type[$k]=PDO::PARAM_INT;
					} else {
						$type[$k]=PDO::PARAM_STR;
					}
					$sql.=$whereArray[$k]["name"]." ".$whereArray[$k]["operator"]." :".$whereArray[$k]["name"].$k;
					if ($k<count($whereArray)-1) {
						$sql.=" ".$whereArray[$k]["and|or"]." ";
					}
				}
				$statement=self::$mysqlLink->prepare($sql);
				foreach ($whereArray as $k => $v) {
					$statement->bindParam(':'.$whereArray[$k]["name"].$k, $whereArray[$k]["value"], $type[$k]);
				}
				try {
					$statement->execute();
					$res=array();
					while ($r=$statement->fetch(PDO::FETCH_ASSOC)) {
						if (is_array($r)) {
							foreach($r as $k => $v) {
								if (!array_key_exists($k, $res)) {
									$res[$k]=array();
								}
								$res[$k][].=(($treatmentFunction!=null)?$treatmentFunction($v):$v);
							}
						}
						unset($r);
					}
					$statement->closeCursor();
					if ($sortParameters["boolean"]==false) {
						return $res;
					} else {
						if (is_array($sortParameters["key"])) {
							$sortParameters["key"]=array_reverse($sortParameters["key"]);
							foreach($sortParameters["key"] as $k=>$v) {
								$res=sortOn($res, $sortParameters["key"][$k], $sortParameters["sort_flags"], $sortParameters["boolean_remap_numerical"]);
							}
						}
						return $res;
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function selectOnFieldsWhere($tableAndFields, $whereArray, $treatmentFunction, $sortParameters=array("boolean"=>false, "key"=>null, "sort_flags"=>null, "boolean_remap_numerical"=>false)) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $tableAndFields[0])) {
				$intTypes=array("tinyint","smallint","mediumint","int","bigint");
				$sql="select ";
				for ($i=0; $i<count($tableAndFields[1]); $i++) {
					$sql.=$tableAndFields[1][$i];
					if ($i<count($tableAndFields[1])-1) {
						$sql.=', ';
					}
				}
				$sql.=" from ".$tableAndFields[0]." where ";
				$type=array();
				foreach ($whereArray as $k => $v) {
					$statement=self::$mysqlLink->prepare("select data_type from information_schema.columns where table_schema='".self::$mysqlBase."' and table_name=:table and column_name=:column");
					$statement->bindParam(':table', $tableAndFields[0], PDO::PARAM_STR);
					$statement->bindParam(':column', $whereArray[$k]["name"], PDO::PARAM_STR);
					try {
						$statement->execute();
						$r=$statement->fetch(PDO::FETCH_ASSOC);
						$type[$k]=$r['data_type'];
						$statement->closeCursor();
					} catch (PDOException $e) {
						return array("message"=>"ko","error"=>$e->getMessage());
					}
					if (strlen($type[$k])>0 && in_array(strtolower($type[$k]),$intTypes)) {
						$type[$k]=PDO::PARAM_INT;
					} else {
						$type[$k]=PDO::PARAM_STR;
					}
					$sql.=$whereArray[$k]["name"]." ".$whereArray[$k]["operator"]." :".$whereArray[$k]["name"].$k;
					if ($k<count($whereArray)-1) {
						$sql.=" ".$whereArray[$k]["and|or"]." ";
					}
				}
				$statement=self::$mysqlLink->prepare($sql);
				foreach ($whereArray as $k => $v) {
					$statement->bindParam(':'.$whereArray[$k]["name"].$k, $whereArray[$k]["value"], $type[$k]);
				}
				try {
					$statement->execute();
					$res=array();
					while ($r=$statement->fetch(PDO::FETCH_ASSOC)) {
						if (is_array($r)) {
							foreach($r as $k => $v) {
								if (!array_key_exists($k, $res)) {
									$res[$k]=array();
								}
								$res[$k][].=(($treatmentFunction!=null)?$treatmentFunction($v):$v);
							}
						}
						unset($r);
					}
					$statement->closeCursor();
					if ($sortParameters["boolean"]==false) {
						return $res;
					} else {
						if (is_array($sortParameters["key"])) {
							$sortParameters["key"]=array_reverse($sortParameters["key"]);
							foreach($sortParameters["key"] as $k=>$v) {
								$res=sortOn($res, $sortParameters["key"][$k], $sortParameters["sort_flags"], $sortParameters["boolean_remap_numerical"]);
							}
						}
						return $res;
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function selectAllPlusString($table, $string, $treatmentFunction, $sortParameters=array("boolean"=>false, "key"=>null, "sort_flags"=>null, "boolean_remap_numerical"=>false)) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$sql="select * from ".$table." ";
				$sql.=$string;
				$statement=self::$mysqlLink->prepare($sql);
				try {
					$statement->execute();
					$res=array();
					while ($r=$statement->fetch(PDO::FETCH_ASSOC)) {
						foreach($r as $k => $v) {
							if (!array_key_exists($k, $res)) {
								$res[$k]=array();
							}
							$res[$k][].=(($treatmentFunction!=null)?$treatmentFunction($v):$v);
						}
						unset($r);
					}
					$statement->closeCursor();
					if ($sortParameters["boolean"]==false) {
						return $res;
					} else {
						if (is_array($sortParameters["key"])) {
							$sortParameters["key"]=array_reverse($sortParameters["key"]);
							foreach($sortParameters["key"] as $k=>$v) {
								$res=sortOn($res, $sortParameters["key"][$k], $sortParameters["sort_flags"], $sortParameters["boolean_remap_numerical"]);
							}
						}
						return $res;
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function selectOnFieldsPlusString($tableAndFields, $string, $treatmentFunction, $sortParameters=array("boolean"=>false, "key"=>null, "sort_flags"=>null, "boolean_remap_numerical"=>false)) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $tableAndFields[0])) {
				$sql="select ";
				for ($i=0; $i<count($tableAndFields[1]); $i++) {
					$sql.=$tableAndFields[1][$i];
					if ($i<count($tableAndFields[1])-1) {
						$sql.=', ';
					}
				}
				$sql.=" from ".$tableAndFields[0]." ";
				$sql.=$string;
				$statement=self::$mysqlLink->prepare($sql);
				try {
					$statement->execute();
					$res=array();
					while ($r=$statement->fetch(PDO::FETCH_ASSOC)) {
						if (is_array($r)) {
							foreach($r as $k => $v) {
								if (!array_key_exists($k, $res)) {
									$res[$k]=array();
								}
								$res[$k][].=(($treatmentFunction!=null)?$treatmentFunction($v):$v);
							}
						}
						unset($r);
					}
					$statement->closeCursor();
					if ($sortParameters["boolean"]==false) {
						return $res;
					} else {
						if (is_array($sortParameters["key"])) {
							$sortParameters["key"]=array_reverse($sortParameters["key"]);
							foreach($sortParameters["key"] as $k=>$v) {
								$res=sortOn($res, $sortParameters["key"][$k], $sortParameters["sort_flags"], $sortParameters["boolean_remap_numerical"]);
							}
						}
						return $res;
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function insertAll($table, $keyValueArray) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$intTypes=array("tinyint","smallint","mediumint","int","bigint");
				$ins="insert into ".$table." (";
				$select="select id from ".$table." where ";
				$i=0;
				foreach ($keyValueArray as $k => $v) {
					$ins.=$k;
					if ($i<count($keyValueArray)-1) {
						$ins.=", ";
					}
					$i++;
				}
				$ins.=") values (";
				$i=0;
				$type=array();
				foreach ($keyValueArray as $k => $v) {
					$statement=self::$mysqlLink->prepare("select data_type from information_schema.columns where table_schema='".self::$mysqlBase."' and table_name=:table and column_name=:column");
					$statement->bindParam(':table', $table, PDO::PARAM_STR);
					$statement->bindParam(':column', $k, PDO::PARAM_STR);
					try {
						$statement->execute();
						$r=$statement->fetch(PDO::FETCH_ASSOC);
						$type[$k]=$r['data_type'];
						$statement->closeCursor();
					} catch (PDOException $e) {
						return array("message"=>"ko","error"=>$e->getMessage());
					}
					$quotes=false;
					if (strlen($type[$k])>0 && in_array(strtolower($type[$k]),$intTypes)) {
						$type[$k]=PDO::PARAM_INT;
					} else {
						$type[$k]=PDO::PARAM_STR;
					}
					$ins.=':'.$k;
					$select.=$k."=:".$k;
					if ($i<count($keyValueArray)-1) {
						$ins.=", ";
						$select.=" and ";
					}
					$i++;
				}
				$ins.=")";
				$statement=self::$mysqlLink->prepare($ins);
				$statementId=self::$mysqlLink->prepare($select);
				foreach ($keyValueArray as $k => $v) {
					$statement->bindParam(':'.$k, $keyValueArray[$k], $type[$k]);
					$statementId->bindParam(':'.$k, $keyValueArray[$k], $type[$k]);
				}
				try {
					$req=$statement->execute();
					if (!$req) {
						return array("message"=>"ko","error"=>implode(", ", $statement->errorInfo()));
					} else {
						try {
							$req=$statementId->execute();
							if (!$req) {
								return array("message"=>"ko","error"=>implode(", ", $statementId->errorInfo()));
							} else {
								$id="";
								while ($rId=$statementId->fetch(PDO::FETCH_ASSOC)) {
									$id=$rId['id'];
								}
								$statementId->closeCursor();
								if ((int)$id>0) {
									return array("message"=>"ok","id"=>$id);
								} else {
									return array("message"=>"ok","id"=>null,"error"=>"erreur inconnue");
								}
							}
						} catch (PDOException $e) {
							return array("message"=>"ko","id"=>null,"error"=>$e->getMessage());
						}
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","id"=>null,"error"=>$e->getMessage());
				}
			} else {
				return array("message"=>"ko","id"=>null,"error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function updateWhere($table, $keyValueArray, $whereArray, $additionnalString="") {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$intTypes=array("tinyint","smallint","mediumint","int","bigint");
				$upd="update ".$table." set ";
				$type=array();
				$uniqid=array();
				foreach ($keyValueArray as $k => $v) {
					if (substr($k, 0, 5)!="where") {
						$statement=self::$mysqlLink->prepare("select data_type from information_schema.columns where table_schema='".self::$mysqlBase."' and table_name=:table and column_name=:column");
						$statement->bindParam(':table', $table, PDO::PARAM_STR);
						$statement->bindParam(':column', $k, PDO::PARAM_STR);
						try {
							$statement->execute();
							$r=$statement->fetch(PDO::FETCH_ASSOC);
							$type[$k]=$r['data_type'];
							$statement->closeCursor();
						} catch (PDOException $e) {
							return array("message"=>"ko","error"=>$e->getMessage());
						}
						$quotes=false;
						if (strlen($type[$k])>0 && in_array(strtolower($type[$k]),$intTypes)) {
							$type[$k]=PDO::PARAM_INT;
						} else {
							$type[$k]=PDO::PARAM_STR;
						}
						$uniqid[$k]=uniqid();
						$upd.=$k."=:".$k.$uniqid[$k];
						$upd.=", ";
					}
				}
				$upd=substr($upd, 0, strlen($upd)-2)." where ";
				for ($i=0; $i<count($whereArray); $i++) {
					if (array_key_exists($i, $whereArray) && is_array($whereArray[$i])) {
						$statement=self::$mysqlLink->prepare("select data_type from information_schema.columns where table_schema='".self::$mysqlBase."' and table_name=:table and column_name=:column");
						$statement->bindParam(':table', $table, PDO::PARAM_STR);
						$statement->bindParam(':column', $whereArray[$i]["name"], PDO::PARAM_STR);
						$typeWhere=array();
						try {
							$r=$statement->execute();
							$typeWhere[$i]=$r[0];
							$statement->closeCursor();
						} catch (PDOException $e) {
							error_log(1,0);
							return array("message"=>"ko","error"=>$e->getMessage());
						}
						$upd.=$whereArray[$i]["name"]." ".$whereArray[$i]["operator"]." :".$whereArray[$i]["name"];
						if ($whereArray[$i]["and|or"]!==null) {
							$upd.=" ".$whereArray[$i]["and|or"]." ";
						}
					} else {
						return array("message"=>"ko","error"=>"whereClauseBadlyWritten");
					}
				}
				$upd.=" ".$additionnalString;
				echo ($upd);
				$statement=self::$mysqlLink->prepare($upd);
				foreach ($keyValueArray as  $k => $v) {
					$statement->bindParam(':'.$k.$uniqid[$k], $keyValueArray[$k], $type[$k]);
				}
				for ($i=0; $i<count($whereArray); $i++) {
					if (array_key_exists($i, $whereArray) && is_array($whereArray[$i])) {
						$statement->bindParam(":".$whereArray[$i]["name"], $whereArray[$i]["value"], $typeWhere[$i]);
					} else {
						return array("message"=>"ko","error"=>"whereClauseBadlyWritten");
					}
				}
				try {
					$req=$statement->execute();
					if (!$req) {
						return array("message"=>"ko","error"=>implode(", ", $statement->errorInfo()));
					} else {
						$statement->closeCursor();
						return array("message"=>"ok");
					}
				} catch (PDOException $e) {
					error_log(2,0);
					return array("message"=>"ko","error"=>$e->getMessage());
				}
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function updatePlusString($table, $keyValueArray, $plus) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$intTypes=array("tinyint","smallint","mediumint","int","bigint");
				$upd="update ".$table." set ";
				$type=array();
				$uniqid=array();
				foreach ($keyValueArray as $k => $v) {
					if (substr($k, 0, 5)!="where") {
						$statement=self::$mysqlLink->prepare("select data_type from information_schema.columns where table_schema='".self::$mysqlBase."' and table_name=:table and column_name=:column");
						$statement->bindParam(':table', $table, PDO::PARAM_STR);
						$statement->bindParam(':column', $k, PDO::PARAM_STR);
						try {
							$statement->execute();
							$r=$statement->fetch(PDO::FETCH_ASSOC);
							$type[$k]=$r['data_type'];
							$statement->closeCursor();
						} catch (PDOException $e) {
							return array("message"=>"ko","error"=>$e->getMessage());
						}
						$quotes=false;
						if (strlen($type[$k])>0 && in_array(strtolower($type[$k]),$intTypes)) {
							$type[$k]=PDO::PARAM_INT;
						} else {
							$type[$k]=PDO::PARAM_STR;
						}
						$uniqid[$k]=uniqid();
						$upd.=$k."=:".$k.$uniqid[$k];
						$upd.=", ";
					}
				}
				$upd=substr($upd, 0, strlen($upd)-2)." where ".$plus;
				$statement=self::$mysqlLink->prepare($upd);
				foreach ($keyValueArray as  $k => $v) {
					$statement->bindParam(':'.$k.$uniqid[$k], $keyValueArray[$k], $type[$k]);
				}
				try {
					$req=$statement->execute();
					if (!$req) {
						return array("message"=>"ko","error"=>implode(", ", $statement->errorInfo()));
					} else {
						$statement->closeCursor();
						return array("message"=>"ok");
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function deleteWhereId($table, $id) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$statement=self::$mysqlLink->prepare("delete from ".$table." where id=:id");
				$statement->bindParam(':id', $id, PDO::PARAM_INT);
				try {
					$req=$statement->execute();
					if (!$req) {
						return array("message"=>"ko","error"=>implode(", ", $statement->errorInfo()));
					} else {
						$statement->closeCursor();
						return array("message"=>"ok");
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function deleteWhere($table, $whereArray) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$intTypes=array("tinyint","smallint","mediumint","int","bigint");
				$sql="delete from ".$table." where ";
				$type=array();
				foreach ($whereArray as $k => $v) {
					$statement=self::$mysqlLink->prepare("select data_type from information_schema.columns where table_schema='".self::$mysqlBase."' and table_name=:table and column_name=:column");
					$statement->bindParam(':table', $table, PDO::PARAM_STR);
					$statement->bindParam(':column', $whereArray[$k]["name"], PDO::PARAM_STR);
					try {
						$statement->execute();
						$r=$statement->fetch(PDO::FETCH_ASSOC);
						$type[$k]=$r['data_type'];
						$statement->closeCursor();
					} catch (PDOException $e) {
						return array("message"=>"ko","error"=>$e->getMessage());
					}
					if (strlen($type[$k])>0 && in_array(strtolower($type[$k]),$intTypes)) {
						$type[$k]=PDO::PARAM_INT;
					} else {
						$type[$k]=PDO::PARAM_STR;
					}
					$sql.=$whereArray[$k]["name"]." ".$whereArray[$k]["operator"]." :".$whereArray[$k]["name"].$k;
					if ($k<count($whereArray)-1) {
						$sql.=" ".$whereArray[$k]["and|or"]." ";
					}
				}
				$statement=self::$mysqlLink->prepare($sql);
				foreach ($whereArray as $k => $v) {
					$statement->bindParam(':'.$whereArray[$k]["name"].$k, $whereArray[$k]["value"], $type[$k]);
				}
				try {
					$req=$statement->execute();
					if (!$req) {
						return array("message"=>"ko","error"=>implode(", ", $statement->errorInfo()));
					} else {
						$statement->closeCursor();
						return array("message"=>"ok");
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function deleteWhereString($table, $whereString) {
		if (self::$statusReturn[0]) {
			if (!preg_match('/[^\\\] /', $table)) {
				$sql="delete from ".$table." where ".$whereString;
				$statement=self::$mysqlLink->prepare($sql);
				try {
					$req=$statement->execute();
					if (!$req) {
						return array("message"=>"ko","error"=>implode(", ", $statement->errorInfo()));
					} else {
						$statement->closeCursor();
						return array("message"=>"ok");
					}
				} catch (PDOException $e) {
					return array("message"=>"ko","error"=>$e->getMessage());
				}
			} else {
				return array("message"=>"ko","error"=>"tableStringContainingUnescapedSpaces");
			}
		} else {
			return self::$statusReturn;
		}
	}
	public static function closeMysql() {
		self::$mysqlLink=null;
	}
}
?>